#define __MAIN
#include "mp3info.h"
#undef __MAIN
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <libgen.h>
#include "textfunc.h"

#define MAXDIRS 10

char *glftpd_path=NULL; 
char *dbfile=NULL;
char *genre_dir=NULL;
char *alpha_dir=NULL;
char *group_dir=NULL;
char *link_prefix=NULL;
char dirs[MAXDIRS][256]; // Max of MAXDIRS dirs each of them max of 256chars long ... enough ?

FILE *dbfp;

int exists(char *dir) {
  struct stat statbuf;
  if (stat(dir, &statbuf) == 0) return 1;
  return 0;
}

void do_mp3db(char *filename) {
  char *temp;
  temp = (char *) malloc(strlen(filename) + 5);
  sprintf(temp, "%s\n", filename+strlen(glftpd_path));
  fputs(temp, dbfp);
  free(temp);
}

void do_mp3link(char *src_dirname, char *genre) {
  char *link_src_path, *link_dst_path, *dst_dirname, *releasename, *groupname, *temp;

  temp = strdup(src_dirname);
  releasename = basename(temp);

/*** LINK GENRE ***/
  if (genre_dir) {
    dst_dirname = (char *) malloc(strlen(src_dirname) + strlen(genre_dir));
    sprintf(dst_dirname, "%s/%s", genre_dir, genre);
    if (!exists(dst_dirname))
      if (mkdir(dst_dirname, 0777))
        return;
    link_dst_path = (char *) malloc(strlen(releasename) + strlen(genre_dir) + 10);
    sprintf(link_dst_path, "%s/%s", dst_dirname, releasename);
    if (!exists(link_dst_path)) {
      link_src_path = (char *) malloc(strlen(src_dirname));
      sprintf(link_src_path, "%s%s", link_prefix, src_dirname+strlen(glftpd_path));
      symlink(link_src_path, link_dst_path);
      free(link_src_path);
    }
    free(link_dst_path);
    free(dst_dirname);
  }

/*** LINK ALPHA ***/
  if (alpha_dir) {
    dst_dirname = (char *) malloc(strlen(src_dirname) + strlen(alpha_dir));
    sprintf(dst_dirname, "%s/%c", alpha_dir, toupper(releasename[0]));
    if (!exists(dst_dirname))
      if (mkdir(dst_dirname, 0777))
        return;
    link_dst_path = (char *) malloc(strlen(releasename) + strlen(alpha_dir) + 10);
    sprintf(link_dst_path, "%s/%s", dst_dirname, releasename);
    if (!exists(link_dst_path)) {
      link_src_path = (char *) malloc(strlen(src_dirname));
      sprintf(link_src_path, "%s%s", link_prefix, src_dirname+strlen(glftpd_path));
      symlink(link_src_path, link_dst_path);
      free(link_src_path);
    }
    free(link_dst_path);
    free(dst_dirname);
  }

/*** LINK RLS_GROUP ***/
  if (group_dir) {
    dst_dirname = (char *) malloc(strlen(src_dirname) + strlen(group_dir));
    groupname = strrchr(releasename, '-');
    if (!groupname)
      groupname="UNKNOWN";
    else
      groupname++;
    sprintf(dst_dirname, "%s/%s", group_dir, groupname);
    if (!exists(dst_dirname))
      if (mkdir(dst_dirname, 0777))
        return;
    link_dst_path = (char *) malloc(strlen(releasename) + strlen(group_dir) + 10);
    sprintf(link_dst_path, "%s/%s", dst_dirname, releasename);
    if (!exists(link_dst_path)) {
      link_src_path = (char *) malloc(strlen(src_dirname));
      sprintf(link_src_path, "%s%s", link_prefix, src_dirname+strlen(glftpd_path));
      symlink(link_src_path, link_dst_path);
      free(link_src_path);
    }
    free(link_dst_path);
    free(dst_dirname);
  }
  free(temp);
}

void do_file(char *filename) {
   FILE  *fp;
   mp3info mp3;
   char genre[40], extention[5];

   strcpy(extention, filename+strlen(filename)-4); extention[4]=0;
   if ( strcasecmp(extention, ".mp3") != 0 ) return;

   fp = fopen(filename, "r");
   if (fp) {
     memset(&mp3, 0, sizeof(mp3info));
     mp3.filename=filename;
     mp3.file=fp;
     get_mp3_info(&mp3, SCAN_QUICK, 1);
     if (mp3.header_isvalid) {
       text_genre(mp3.id3.genre, genre);
       do_mp3db(filename);
       do_mp3link(dirname(filename), genre);
     }
     fclose(fp);
   }
}

void do_dir(char *path) {
  DIR *dir;
  struct dirent *direntry;
  struct stat statbuf;
  char buffer[1024]; /* This is static buffer instead of a dynamic malloc()'ed one: for performance gain */

  dir = opendir(path);
  if (!dir) return;

  while ( (direntry = readdir(dir)) ) {

    if ( !strncmp(direntry->d_name, ".",  1) ||
         !strncmp(direntry->d_name, "..", 2) ) continue;

    sprintf(buffer, "%s/%s", path, direntry->d_name);

    if ( stat(buffer, &statbuf) != -1 ) {
      if ( S_ISDIR(statbuf.st_mode) )
        do_dir(buffer);
      else
        do_file(buffer);
    }
  }

  closedir(dir);
}

char *trim(char *str) {
  char *ibuf, *obuf;
  if (str) {
    for (ibuf = obuf = str; *ibuf; ) {
      while (*ibuf && (isspace (*ibuf)))
        ibuf++;
      if (*ibuf && (obuf != str))
        *(obuf++) = ' ';
      while (*ibuf && (!isspace (*ibuf)))
        *(obuf++) = *(ibuf++);
    }
    *obuf = 0;
  }
  return (str);
}

int main(int argc, char **argv) {
  FILE *configfp;
  char buffer[512];
  int dircount;
  if (argc != 2) {
    printf("Usage: %s <config file>\n", argv[0]);
    return 1;
  }

  configfp = fopen(argv[1], "r");
  if (configfp) {
    dircount=0;
    while (fgets(buffer,sizeof(buffer), configfp)) {
      if (buffer[0] == '#' || buffer[0] == ' ' || buffer[0] == '\t' || buffer[0] == '\n') continue;
      if (!strncmp(buffer, "GLFTPD_PATH", 11)) {
        glftpd_path = (char *)malloc(strlen(buffer));
        sprintf(glftpd_path, "%s", buffer+11);
 	trim(glftpd_path);
      } else
      if (!strncmp(buffer, "DATABASE_FILE", 13)) {
        dbfile = (char *)malloc(strlen(buffer));
        sprintf(dbfile, "%s", buffer+13);
 	trim(dbfile);
      } else
      if (!strncmp(buffer, "GENRE_DIR", 9)) {
        genre_dir = (char *)malloc(strlen(buffer));
        sprintf(genre_dir, "%s", buffer+9);
 	trim(genre_dir);
      } else
      if (!strncmp(buffer, "ALPHA_DIR", 9)) {
        alpha_dir = (char *)malloc(strlen(buffer));
        sprintf(alpha_dir, "%s", buffer+9);
 	trim(alpha_dir);
      } else
      if (!strncmp(buffer, "GROUP_DIR", 9)) {
        group_dir = (char *)malloc(strlen(buffer));
        sprintf(group_dir, "%s", buffer+9);
 	trim(group_dir);
      } else
      if (!strncmp(buffer, "LINK_PREFIX", 11)) {
        link_prefix = (char *)malloc(strlen(buffer));
        sprintf(link_prefix, "%s", buffer+11);
 	trim(link_prefix);
      } else
      if (!strncmp(buffer, "MP3_DIR", 7)) {
        sprintf(dirs[dircount], "%s", buffer+7);
        trim(dirs[dircount]);
        dircount++;
      } else
      printf("%s", buffer);
    }
    if (!link_prefix) { printf("Option 'LINK_PREFIX' is missing!!!\n"); return 1; }
    if (!dbfile) { printf("Option 'DATABASE_FILE' is missing!!!\n"); return 1; }
    if (!glftpd_path) { printf("Option 'GLFTPD_PATH' is missing!!!\n"); return 1; }
    dircount--;
    if (dircount >= 0) {
      sprintf(buffer,"/bin/rm -rf %s/*", genre_dir); system(buffer);
      sprintf(buffer,"/bin/rm -rf %s/*", alpha_dir); system(buffer);
      sprintf(buffer,"/bin/rm -rf %s/*", group_dir); system(buffer);
      dbfp = fopen(dbfile, "w");
      while (dircount >= 0) {
        do_dir(dirs[dircount]);
        dircount--;
      }
      fclose(dbfp);
    }
    fclose(configfp);
  } else {
    printf("Cannot read configuration file %s\n", argv[1]);
    return 1;
  }
  return 0;
}
